"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getExportWebpackConfig = getExportWebpackConfig;

var _path = _interopRequireDefault(require("path"));

var _htmlPlugin = require("./htmlPlugin");

var _shared = require("./shared");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function getExportWebpackConfig(cosmosConfig, userWebpack) {
  var baseWebpackConfig = (0, _shared.getUserWebpackConfig)(cosmosConfig, userWebpack);
  return _objectSpread({}, baseWebpackConfig, {
    entry: getEntry(),
    output: getOutput(cosmosConfig),
    module: _objectSpread({}, baseWebpackConfig.module, {
      rules: getRules(baseWebpackConfig)
    }),
    resolve: (0, _shared.resolveLocalReactDeps)(cosmosConfig, baseWebpackConfig),
    plugins: getPlugins(cosmosConfig, baseWebpackConfig, userWebpack)
  });
}

function getEntry() {
  // The React devtools hook needs to be imported before any other module that
  // might import React
  var devtoolsHook = (0, _shared.resolveClientPath)('reactDevtoolsHook');
  var clientIndex = (0, _shared.resolveClientPath)('index');
  return [devtoolsHook, clientIndex];
}

function getOutput(_ref) {
  var exportPath = _ref.exportPath,
      publicUrl = _ref.publicUrl;
  var filename = '[name].js';
  return {
    // Most paths are created using forward slashes regardless of the OS for
    // consistency, but this one needs to have backslashes on Windows!
    path: _path.default.join(exportPath, publicUrl),
    filename: filename,
    publicPath: publicUrl
  };
}

function getRules(baseWebpackConfig) {
  var existingRules = baseWebpackConfig.module && baseWebpackConfig.module.rules || [];
  return [].concat(_toConsumableArray(existingRules), [(0, _shared.getUserDepsLoaderRule)()]);
}

function getPlugins(cosmosConfig, baseWebpackConfig, userWebpack) {
  var existingPlugins = baseWebpackConfig.plugins || [];
  var globalsPlugin = (0, _shared.getGlobalsPlugin)(cosmosConfig, userWebpack, false);
  var noEmitErrorsPlugin = new userWebpack.NoEmitOnErrorsPlugin();
  return (0, _htmlPlugin.ensureHtmlWebackPlugin)(cosmosConfig, [].concat(_toConsumableArray(existingPlugins), [globalsPlugin, noEmitErrorsPlugin]));
}