"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getUserWebpackConfig = getUserWebpackConfig;
exports.getBaseWebpackConfig = getBaseWebpackConfig;
exports.resolveClientPath = resolveClientPath;
exports.getUserDepsLoaderRule = getUserDepsLoaderRule;
exports.resolveLocalReactDeps = resolveLocalReactDeps;
exports.getGlobalsPlugin = getGlobalsPlugin;
exports.hasPlugin = hasPlugin;
exports.isInstanceOfPlugin = isInstanceOfPlugin;
exports.getNodeEnv = getNodeEnv;

var _path = _interopRequireDefault(require("path"));

var _resolveFrom = _interopRequireDefault(require("resolve-from"));

var _yargs = require("yargs");

var _fs = require("../../../shared/fs");

var _webpack = require("../cosmosConfig/webpack");

var _default = require("./default");

var _module = require("./module");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function getUserWebpackConfig(cosmosConfig, userWebpack) {
  var baseWebpackConfig = getBaseWebpackConfig(cosmosConfig, userWebpack);

  var _createWebpackCosmosC = (0, _webpack.createWebpackCosmosConfig)(cosmosConfig),
      overridePath = _createWebpackCosmosC.overridePath;

  if (!overridePath || !(0, _fs.moduleExists)(overridePath)) {
    return baseWebpackConfig;
  }

  var relPath = _path.default.relative(process.cwd(), overridePath);

  console.log("[Cosmos] Overriding webpack config at ".concat(relPath));
  var webpackOverride = (0, _module.getDefaultExport)((0, _fs.requireModule)(overridePath));
  return webpackOverride(baseWebpackConfig, getNodeEnv());
}

function getBaseWebpackConfig(cosmosConfig, userWebpack) {
  var rootDir = cosmosConfig.rootDir;

  var _createWebpackCosmosC2 = (0, _webpack.createWebpackCosmosConfig)(cosmosConfig),
      configPath = _createWebpackCosmosC2.configPath;

  if (!configPath || !(0, _fs.moduleExists)(configPath)) {
    console.log('[Cosmos] Using default webpack config');
    return (0, _default.getDefaultWebpackConfig)(userWebpack, rootDir);
  }

  var relPath = _path.default.relative(process.cwd(), configPath);

  console.log("[Cosmos] Using webpack config found at ".concat(relPath));
  var userConfigExport = (0, _module.getDefaultExport)((0, _fs.requireModule)(configPath));
  return typeof userConfigExport === 'function' ? userConfigExport(_yargs.argv.env || getNodeEnv(), _yargs.argv) : userConfigExport;
}

function resolveClientPath(relPath) {
  return require.resolve("../client/".concat(relPath));
}

function getUserDepsLoaderRule() {
  return {
    loader: require.resolve('./userDepsLoader'),
    include: resolveClientPath('userDeps')
  };
}

function resolveLocalReactDeps(cosmosConfig, baseWebpackConfig) {
  var rootDir = cosmosConfig.rootDir;

  var reactPath = _resolveFrom.default.silent(rootDir, 'react');

  if (!reactPath) throw new Error("[Cosmos] Local dependency not found: react");

  var reactDomPath = _resolveFrom.default.silent(rootDir, 'react-dom');

  if (!reactDomPath) throw new Error("[Cosmos] Local dependency not found: react-dom");
  var _baseWebpackConfig$re = baseWebpackConfig.resolve,
      resolve = _baseWebpackConfig$re === void 0 ? {} : _baseWebpackConfig$re;
  var _resolve$alias = resolve.alias,
      alias = _resolve$alias === void 0 ? {} : _resolve$alias;
  return _objectSpread({}, resolve, {
    alias: _objectSpread({}, alias, {
      react: _path.default.dirname(reactPath),
      'react-dom': _path.default.dirname(reactDomPath)
    })
  });
}

function getGlobalsPlugin(_ref, userWebpack, devServerOn) {
  var publicUrl = _ref.publicUrl;
  var cleanPublicUrl = removeTrailingSlash(publicUrl);
  return new userWebpack.DefinePlugin({
    // "if (__DEV__)" blocks get stripped when compiling a static export build
    __DEV__: JSON.stringify(devServerOn),
    'process.env': {
      NODE_ENV: JSON.stringify(getNodeEnv()),
      PUBLIC_URL: JSON.stringify(cleanPublicUrl)
    }
  });
}

function hasPlugin(plugins, pluginName) {
  return plugins && plugins.filter(function (p) {
    return isInstanceOfPlugin(p, pluginName);
  }).length > 0;
}

function isInstanceOfPlugin(plugin, constructorName) {
  return plugin.constructor && plugin.constructor.name === constructorName;
}

function removeTrailingSlash(url) {
  return url.replace(/\/$/, '');
}

function getNodeEnv() {
  // Disallow non dev/prod environments, like "test" inside Jest, because
  // they are not supported by webpack
  return process.env.NODE_ENV === 'production' ? 'production' : 'development';
}